searchData={"items":[{"type":"module","title":"LiveReact","doc":"See READ.md for installation instructions and examples.","ref":"LiveReact.html"},{"type":"function","title":"LiveReact.react/1","doc":"Render a React component.","ref":"LiveReact.html#react/1"},{"type":"behaviour","title":"LiveReact.SSR","doc":"A behaviour for rendering React components server-side.\n\nTo define a custom renderer, change the application config in `config.exs`:\n\n    config :live_react, ssr_module: MyCustomSSRModule\n\nExposes a telemetry span for each render under key `[:live_react, :ssr]`","ref":"LiveReact.SSR.html"},{"type":"function","title":"LiveReact.SSR.render/2","doc":"","ref":"LiveReact.SSR.html#render/2"},{"type":"callback","title":"LiveReact.SSR.render/2","doc":"","ref":"LiveReact.SSR.html#c:render/2"},{"type":"type","title":"LiveReact.SSR.component_name/0","doc":"","ref":"LiveReact.SSR.html#t:component_name/0"},{"type":"type","title":"LiveReact.SSR.props/0","doc":"","ref":"LiveReact.SSR.html#t:props/0"},{"type":"type","title":"LiveReact.SSR.render_response/0","doc":"A render response which should have shape\n\n%{\n  html: string,\n}","ref":"LiveReact.SSR.html#t:render_response/0"},{"type":"module","title":"LiveReact.SSR.NodeJS","doc":"Implements SSR by using `NodeJS` package.\n\nUnder the hood, it invokes \"render\" function exposed by `server.js` file.\nYou can see how `server.js` is created by looking at `assets.deploy` command\nand `package.json` build-server script.","ref":"LiveReact.SSR.NodeJS.html"},{"type":"function","title":"LiveReact.SSR.NodeJS.render/2","doc":"","ref":"LiveReact.SSR.NodeJS.html#render/2"},{"type":"function","title":"LiveReact.SSR.NodeJS.server_path/0","doc":"","ref":"LiveReact.SSR.NodeJS.html#server_path/0"},{"type":"extras","title":"README","doc":"[![Hex.pm](https://img.shields.io/hexpm/v/live_react.svg)](https://hex.pm/packages/live_react)\n[![Hexdocs.pm](https://img.shields.io/badge/docs-hexdocs.pm-purple)](https://hexdocs.pm/live_react)\n[![GitHub](https://img.shields.io/github/stars/mrdotb/live_react?style=social)](https://github.com/mrdotb/live_vue)\n\n# LiveReact\n\nReact inside Phoenix LiveView.\n\n![logo](https://github.com/mrdotb/live_react/blob/main/logo.svg?raw=true)","ref":"readme.html"},{"type":"extras","title":"Features - README","doc":"- ⚡ **End-To-End Reactivity** with LiveView\n- 🔋 **Server-Side Rendered** (SSR) React\n- 🦄 **Tailwind** Support\n- 💀 **Dead View** Support\n- 🐌 **Lazy-loading** React Components","ref":"readme.html#features"},{"type":"extras","title":"Resources - README","doc":"- [Demo](https://live-react-examples.fly.dev/simple)\n- [HexDocs](https://hexdocs.pm/live_react)\n- [HexPackage](https://hex.pm/packages/live_react)\n- [Phoenix LiveView](https://github.com/phoenixframework/phoenix_live_view)\n- [Development](/guides/development.md)\n- [SSR](/guides/ssr.md)\n- [Deployment](/guides/deployment.md)","ref":"readme.html#resources"},{"type":"extras","title":"Why LiveReact - README","doc":"Phoenix LiveView enables rich, real-time user experiences with server-rendered HTML.\nIt works by communicating any state changes through a websocket and updating the DOM in realtime.\nYou can get a really good user experience without ever needing to write any client side code.\n\nLiveReact builds on top of Phoenix LiveView to allow for easy client side state management while still allowing for communication over the websocket.","ref":"readme.html#why-livereact"},{"type":"extras","title":"Installation - README","doc":"1. Add `live_react` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:live_react, \"~> 0.2.0-rc.0\"}\n  ]\nend\n```\n\n2. Adjust the `setup` in `mix.exs`:\n\n- For Linux/MacOs\n\n```elixir\ndefp aliases do\n    [\n      setup: [\"deps.get\", \"assets.setup\", \"npm install --prefix assets\", \"assets.build\"],\n    ]\nend\n```\n\n- For Windows\n\n```elixir\ndefp aliases do\n    [\n      setup: [\"deps.get\", \"assets.setup\", \"cmd --cd assets npm install\", \"assets.build\"],\n    ]\nend\n```\n\n3. Run the following in your terminal\n\n```bash\nmix deps.get\n```\n\n4. Add `import LiveReact` in `html_helpers/0` inside `/lib/ _web.ex` like so:\n\n```elixir\n# /lib/ _web.ex\n\ndefp html_helpers do\n  quote do\n\n    # ...\n\n    import LiveReact # <-- Add this line\n\n    # ...\n\n  end\nend\n```\n\n5. Create a package.json file at `assets/package.json`\n\n```json\n{\n  \"dependencies\": {\n    \"@mrdotb/live-react\": \"^0.2.1-beta\",\n    \"phoenix\": \"file:../deps/phoenix\",\n    \"phoenix_html\": \"file:../deps/phoenix_html\",\n    \"phoenix_live_view\": \"file:../deps/phoenix_live_view\",\n    \"react\": \"^18.3.1\",\n    \"react-dom\": \"^18.3.1\"\n  }\n}\n```\n\n6. Create a `react` directory inside `assets` and add a `index.js` file\n\n```javascript\n// import and export the react components you want to use in your LiveView\n// ex: import { Simple } from \"./simple\";\n\n// ex: export default { Simple };\nexport default {};\n```\n\n7. Add the following to your `assets/js/app.js` file\n\n```javascript\n...\nimport components from \"../react\";\nimport { getHooks } from  \"@mrdotb/live-react\";\n\nconst hooks = {\n  ...getHooks(components),\n};\n\nlet csrfToken = document\n  .querySelector(\"meta[name='csrf-token']\")\n  .getAttribute(\"content\");\nlet liveSocket = new LiveSocket(\"/live\", Socket, {\n  hooks: hooks, // <-- pass the hooks\n  longPollFallbackMs: 2500,\n  params: { _csrf_token: csrfToken },\n});\n...\n```\n\n7. For tailwind support, make some addition to `content` in the `assets/tailwind.config.js` file\n\n```javascript\ncontent: [\n  ...\n    \"./react/**/*.jsx\", // <- if you are using jsx\n    \"./react/**/*.tsx\" // <- if you are using tsx\n],\n\n```","ref":"readme.html#installation"},{"type":"extras","title":"Examples - README","doc":"Check out the [demo website](https://live-react-examples.fly.dev/simple) on fly.io to see some examples of what you can do with LiveReact.","ref":"readme.html#examples"},{"type":"extras","title":"Credits - README","doc":"I was inspired by the following libraries:\n\n- [LiveVue](https://github.com/Valian/live_vue)\n- [LiveSvelte](https://github.com/woutdp/live_svelte)\n\nI had a need for a similar library for React and so I created LiveReact 👍","ref":"readme.html#credits"},{"type":"extras","title":"Deployment","doc":"# Deployment\n\n[Without SSR](#without-ssr)\n[With SSR](#with-ssr)","ref":"deployment.html"},{"type":"extras","title":"Without SSR - Deployment","doc":"The following steps are needed to deploy to Fly.io. This guide assumes that you'll be using Fly Postgres as your database. Further guidance on how to deploy to Fly.io can be found [here](https://fly.io/docs/elixir/getting-started/).\n\n1. Generate a `Dockerfile`:\n\n```bash\nmix phx.gen.release --docker\n```\n\n2. Modify the generated `Dockerfile` to install `curl`, which is used to install `nodejs` (version 19 or greater), and also add a step to install our `npm` dependencies:\n\n```diff\n# ./Dockerfile\n\n...\n\n# install build dependencies\n- RUN apt-get update -y && apt-get install -y build-essential git \\\n+ RUN apt-get update -y && apt-get install -y build-essential git curl \\\n    && apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n+ # install nodejs for build stage\n+ RUN curl -fsSL https://deb.nodesource.com/setup_19.x | bash - && apt-get install -y nodejs\n\n...\n\nCOPY assets assets\n\n+ # install all npm packages in assets directory\n+ RUN cd /app/assets && npm install\n```\n\nNote: `nodejs` is installed in the build stage. This is because we need `nodejs` to install our `npm` dependencies.\n\n3. Launch your app with the Fly.io CLI:\n\n```bash\nfly launch\n```\n\n4. When prompted to tweak settings, choose `y`:\n\n```bash\n? Do you want to tweak these settings before proceeding? (y/N) y\n```\n\nThis will launch a new window where you can tweak your launch settings. In the database section, choose `Fly Postgres` and enter a name for your database. You may also want to change your database to the development configuration to avoid extra costs. You can leave the rest of the settings as-is unless you want to change them.\n\nDeployment will continue once you hit confirm.\n\n5. Once the deployment completes, run the following command to see your deployed app!\n\n```bash\nfly apps open\n```","ref":"deployment.html#without-ssr"},{"type":"extras","title":"With SSR - Deployment","doc":"See the [SSR guide](/guides/ssr.md) first to setup your project.\n\nThe following steps are needed to deploy to Fly.io. This guide assumes that you'll be using Fly Postgres as your database. Further guidance on how to deploy to Fly.io can be found [here](https://fly.io/docs/elixir/getting-started/).\n\n1. Generate a `Dockerfile`:\n\n```bash\nmix phx.gen.release --docker\n```\n\n2. Modify the generated `Dockerfile` to install `curl`, which is used to install `nodejs` (version 19 or greater), and also add a step to install our `npm` dependencies:\n\n```diff\n# ./Dockerfile\n\n...\n\n# install build dependencies\n- RUN apt-get update -y && apt-get install -y build-essential git \\\n+ RUN apt-get update -y && apt-get install -y build-essential git curl \\\n    && apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n+ # install nodejs for build stage\n+ RUN curl -fsSL https://deb.nodesource.com/setup_19.x | bash - && apt-get install -y nodejs\n\n...\n\nCOPY assets assets\n\n+ # install all npm packages in assets directory\n+ RUN cd /app/assets && npm install\n\n...\n\n# start a new build stage so that the final image will only contain\n# the compiled release and other runtime necessities\nFROM ${RUNNER_IMAGE}\n\nRUN apt-get update -y && \\\n-  apt-get install -y libstdc++6 openssl libncurses5 locales ca-certificates \\\n+  apt-get install -y libstdc++6 openssl libncurses5 locales ca-certificates curl \\\n   && apt-get clean && rm -f /var/lib/apt/lists/*_*\n\n+ # install nodejs for production environment\n+ RUN curl -fsSL https://deb.nodesource.com/setup_19.x | bash - && apt-get install -y nodejs\n\n...\n```\n\nNote: `nodejs` is installed BOTH in the build stage and in the final image. This is because we need `nodejs` to install our `npm` dependencies and also need it when running our app.\n\n3. Launch your app with the Fly.io CLI:\n\n```bash\nfly launch\n```\n\n4. When prompted to tweak settings, choose `y`:\n\n```bash\n? Do you want to tweak these settings before proceeding? (y/N) y\n```\n\nThis will launch a new window where you can tweak your launch settings. In the database section, choose `Fly Postgres` and enter a name for your database. You may also want to change your database to the development configuration to avoid extra costs. You can leave the rest of the settings as-is unless you want to change them.\n\nDeployment will continue once you hit confirm.\n\n5. Once the deployment completes, run the following command to see your deployed app!\n\n```bash\nfly apps open\n```","ref":"deployment.html#with-ssr"},{"type":"extras","title":"Development","doc":"# Development","ref":"development.html"},{"type":"extras","title":"React deduplication - Development","doc":"Because live-react depends on react and react-dom and putting a dependency using file: in package.json does not guarantee deduplication we need to publish the package on npm.\n\nWhen developing live-react locally you should use [Verdaccio](https://verdaccio.org/)\n\n```bash\nnpm i -g verdaccio\nverdaccio\n```\n\nIn another shell\n\n```bash\n# The adduser is only needed once\nnpm adduser --registry http://localhost:4873/\n# from project root publish the package\nnpm publish --registry http://localhost:4873/\n\ncd live_react_examples/assets\nnpm i @mrdotb/live-react@0.x.0 --registry http://localhost:4873/\n```\n\nThis is not the best solution but I could not find a better one yet.","ref":"development.html#react-deduplication"},{"type":"extras","title":"Server Side Rendering (SSR)","doc":"# Server Side Rendering (SSR)\n\n_Disclaimer_ SSR is not a simple topic and there is a lot of issue than can arise depending on what React components you are using. This is a simple implementation that works for the components and library I have tested.","ref":"ssr.html"},{"type":"extras","title":"Project setup - Server Side Rendering (SSR)","doc":"SSR requires Node.js to render the javascript on server side. Add `nodejs` to your mix file.\n\n```elixir\ndefp deps do\n  [\n    {:nodejs, \"~> 3.1\"},\n    ...\n  ]\nend\n```\n\nAdd NodeJs.Supervisor to your `application.ex`\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    ...\n    {NodeJS.Supervisor, [path: LiveReact.SSR.NodeJS.server_path(), pool_size: 4]},\n  ]\nend\n```\n\nA server file is needed copy the `server.mjs` file into your assets\n\n```bash\ncurl https://raw.githubusercontent.com/mrdotb/live-react/main/assets/copy/server.mjs > assets/server.mjs\n```\n\nEdit `config/config.exs` to add a second esbuild process\n\n```elixir\nconfig :esbuild,\n  version: \"0.17.11\",\n  client: [\n    args: ~w(\n        js/app.js\n        --chunk-names=[name]-[hash]\n        --splitting\n        --format=esm\n        --bundle\n        --target=es2020\n        --main-fields=module,main,exports\n        --outdir=../priv/static/assets\n        --external:/fonts/* --external:/images/*\n      ),\n    cd: Path.expand(\"../assets\", __DIR__),\n    env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n  ],\n  server: [\n    args: ~w(\n      js/server.mjs\n      --bundle\n      --platform=node\n      --target=node19\n      --outdir=../priv/react\n    ),\n    cd: Path.expand(\"../assets\", __DIR__),\n    env: %{\"NODE_PATH\" => Path.expand(\"../deps\", __DIR__)}\n  ]\n```\n\nIn your `config/dev.exs` edit the watchers to include a watch on the server esbuild\n\n```elixir\nconfig :live_react_examples, LiveReactExamplesWeb.Endpoint,\n  watchers: [\n    client_esbuild: {Esbuild, :install_and_run, [\n      :client, ~w(--sourcemap=inline --watch),\n    ]},\n    server_esbuild: {Esbuild, :install_and_run, [\n      :server, ~w(--sourcemap=inline --watch),\n    ]}\n  ]\n```\n\nLastly edit the aliases in `mix.exs`\n\n```elixir\ndefp aliases do\n[\n  setup: [\"deps.get\", \"assets.setup\", \"cmd npm install --prefix assets\", \"assets.build\"],\n  \"assets.setup\": [\"tailwind.install --if-missing\", \"esbuild.install --if-missing\"],\n  \"assets.build\": [\"tailwind live_react_examples\", \"esbuild client\", \"esbuild server\"],\n  \"assets.deploy\": [\n    \"tailwind live_react_examples --minify\",\n    \"esbuild client --minify\",\n    \"esbuild server --minify\",\n    \"phx.digest\"\n  ]\n]\nend\n```\n\nFor deployment follow the [SSR deployment guide](/guides/deployment.md#with-ssr)","ref":"ssr.html#project-setup"},{"type":"extras","title":"Change Log","doc":"# Change Log\n\nAll notable changes to this project will be documented in this file.\nSee [Conventional Commits](Https://conventionalcommits.org) for commit guidelines.\n\n<!-- changelog -->\n\n## [v0.2.0-rc.0](https://github.com/mrdotb/live_react/compare/v0.2.0-rc.0...v0.2.0-rc.0) (2024-09-17)","ref":"changelog.html"},{"type":"extras","title":"Features - Change Log","doc":"* Add SSR support","ref":"changelog.html#features"},{"type":"extras","title":"Bug Fixes: - Change Log","doc":"* ssr: remove compiler warning when using live_react without SSR","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.1.0 (2024-06-29) - Change Log","doc":"Initial release","ref":"changelog.html#v0-1-0-2024-06-29"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}